@@ -1,14 +1,40 @@
 from collections import namedtuple
+from dataclasses import dataclass
 from pathlib import Path
 
 import xarray as xr
 
-from ravenpy.models import Ostrich, Raven
+from .base import Ostrich, Raven
+from .commands import BasinIndexCommand
+from .rv import (
+    HRU,
+    LU,
+    RV,
+    RVC,
+    RVH,
+    RVI,
+    RVP,
+    RVT,
+    HRUState,
+    MonthlyAverage,
+    Ost,
+    RavenNcData,
+)
 
-from .rv import RV, RVC, RVI, RVT, MonthlyAverage, Ost, RavenNcData
-from .state import BasinStateVariables, HRUStateVariables
-
-__all__ = ["GR4JCN", "MOHYSE", "HMETS", "HBVEC", "GR4JCN_OST", "MOHYSE_OST", "HMETS_OST", "HBVEC_OST", "get_model"]
+__all__ = [
+    "GR4JCN",
+    "MOHYSE",
+    "HMETS",
+    "HBVEC",
+    "BLENDED",
+    "GR4JCN_OST",
+    "MOHYSE_OST",
+    "HMETS_OST",
+    "HBVEC_OST",
+    "BLENDED_OST",
+    "get_model",
+    "Routing",
+]
 
 nc = RavenNcData
 std_vars = (
@@ -56,7 +82,8 @@ class GR4JCN(Raven):
         )
 
         # Initialize the stores to 1/2 full. Declare the parameters that can be user-modified
-        self.rvc = RVC(soil0=None, soil1=15, basin_state=BasinStateVariables())
+        # TODO: Is this really setting soil0 and soil1 ? I think it needs to be set inside HRUStateVariable.
+        self.rvc = RVC(soil0=None, soil1=15, basin_state=BasinIndexCommand())
         self.rvd = RV(one_minus_CEMANEIGE_X2=None, GR4J_X1_hlf=None)
 
     def derived_parameters(self):
@@ -68,7 +95,7 @@ class GR4JCN(Raven):
             soil0 = self.rvd.GR4J_X1_hlf if self.rvc.soil0 is None else self.rvc.soil0
             soil1 = self.rvc.soil1
 
-            self.rvc.hru_state = HRUStateVariables(soil0=soil0, soil1=soil1)
+            self.rvc.hru_state = HRUState(index=1, soil0=soil0, soil1=soil1)
 
 
 class GR4JCN_OST(Ostrich, GR4JCN):
@@ -106,7 +133,10 @@ class MOHYSE(Raven):
         )
         self.rvt = RVT(**{k: nc() for k in std_vars})
         self.rvi = RVI(evaporation="PET_MOHYSE", rain_snow_fraction="RAINSNOW_DATA")
-        self.rvc = RVC(hru_state=HRUStateVariables(), basin_state=BasinStateVariables())
+        self.rvc = RVC(
+            hru_state=HRUState(),
+            basin_state=BasinIndexCommand(),
+        )
         self.rvd = RV(par_rezi_x10=None)
 
     def derived_parameters(self):
@@ -172,7 +202,7 @@ class HMETS(GR4JCN):
         self.rvp = RV(params=HMETS.params(*((None,) * len(HMETS.params._fields))))
         self.rvt = RVT(**{k: nc() for k in std_vars})
         self.rvi = RVI(evaporation="PET_OUDIN", rain_snow_fraction="RAINSNOW_DATA")
-        self.rvc = RVC(soil0=None, soil1=None, basin_state=BasinStateVariables())
+        self.rvc = RVC(soil0=None, soil1=None, basin_state=BasinIndexCommand())
         self.rvd = RV(
             TOPSOIL_m=None,
             PHREATIC_m=None,
@@ -202,7 +232,7 @@ class HMETS(GR4JCN):
             soil1 = (
                 self.rvd["PHREATIC_hlf"] if self.rvc.soil1 is None else self.rvc.soil1
             )
-            self.rvc.hru_state = HRUStateVariables(soil0=soil0, soil1=soil1)
+            self.rvc.hru_state = HRUState(soil0=soil0, soil1=soil1)
 
 
 class HMETS_OST(Ostrich, HMETS):
@@ -323,9 +353,9 @@ class HBVEC(GR4JCN):
 
         # Default initial conditions if none are given
         if self.rvc.hru_state is None:
-            self.rvc.hru_state = HRUStateVariables(soil2=self.rvc.soil2)
+            self.rvc.hru_state = HRUState(soil2=self.rvc.soil2)
         if self.rvc.basin_state is None:
-            self.rvc.basin_state = BasinStateVariables(qout=(self.rvc.qout,))
+            self.rvc.basin_state = BasinIndexCommand(qout=(self.rvc.qout,))
 
     # TODO: Support index specification and unit changes.
     def _monthly_average(self):
@@ -422,6 +452,236 @@ class HBVEC_OST(Ostrich, HBVEC):
         self._monthly_average()
 
 
+class BLENDED(Raven):
+    identifier = "blended"
+    templates = tuple((Path(__file__).parent / "raven-blended").glob("*.rv?"))
+
+    params = namedtuple(
+        "BLENDEDParams",
+        ", ".join(
+            ["par_x{:02}".format(i) for i in range(1, 36)]
+            + ["par_r{:02}".format(i) for i in range(1, 9)]
+        ),
+    )
+
+    @dataclass
+    class HRU(HRU):
+        land_use_class: str = "FOREST"
+        veg_class: str = "FOREST"
+        soil_profile: str = "DEFAULT_P"
+        aquifer_profile: str = "[NONE]"
+        terrain_class: str = "DEFAULT_T"
+
+    def __init__(self, *args, **kwds):
+        super().__init__(*args, **kwds)
+        self.rvp = RVP(
+            params=BLENDED.params(*((None,) * len(BLENDED.params._fields))),
+            land_use_classes=(
+                LU("FOREST", impermeable_frac=0.0, forest_coverage=0.02345),
+            ),
+        )
+        self.rvh = RVH(hrus=(BLENDED.HRU(),))
+        self.rvt = RVT(**{k: nc() for k in std_vars})
+        self.rvi = RVI(evaporation="PET_OUDIN", rain_snow_fraction="RAINSNOW_HBV")
+        self.rvc = RVC(soil0=None, soil1=None, basin_state=BasinIndexCommand())
+        self.rvd = RV(
+            TOPSOIL_mm=None,
+            PHREATIC_mm=None,
+            TOPSOIL_hlf=None,
+            PHREATIC_hlf=None,
+            POW_X04=None,
+            POW_X11=None,
+            SUM_X09_X10=None,
+            SUM_X13_X14=None,
+            SUM_X24_X25=None,
+        )
+
+    def derived_parameters(self):
+        self.rvd["TOPSOIL_hlf"] = self.rvp.params.par_x29 * 0.5 * 1000.0
+        self.rvd["PHREATIC_hlf"] = self.rvp.params.par_x30 * 0.5 * 1000.0
+        self.rvd["TOPSOIL_mm"] = self.rvp.params.par_x29 * 1000.0
+        self.rvd["PHREATIC_mm"] = self.rvp.params.par_x30 * 1000.0
+        self.rvd["SUM_X09_X10"] = self.rvp.params.par_x10  # + self.rvp.params.par_x09
+        self.rvd["SUM_X13_X14"] = self.rvp.params.par_x14  # + self.rvp.params.par_x13
+        self.rvd["SUM_X24_X25"] = self.rvp.params.par_x25  # + self.rvp.params.par_x24
+        self.rvd[
+            "POW_X04"
+        ] = self.rvp.params.par_x04  # 10.0**self.rvp.params.par_x04  #
+        self.rvd[
+            "POW_X11"
+        ] = self.rvp.params.par_x11  # 10.0**self.rvp.params.par_x11  #
+
+        # Default initial conditions if none are given
+        if self.rvc.hru_state is None:
+            soil0 = (
+                self.rvd["TOPSOIL_hlf"] if self.rvc.soil0 is None else self.rvc.soil0
+            )
+            soil1 = (
+                self.rvd["PHREATIC_hlf"] if self.rvc.soil1 is None else self.rvc.soil1
+            )
+            self.rvc.hru_state = HRUState(soil0=soil0, soil1=soil1)
+
+        self.rvt.raincorrection = self.rvp.params.par_x33
+        self.rvt.snowcorrection = self.rvp.params.par_x34
+
+
+class BLENDED_OST(Ostrich, BLENDED):
+    _p = Path(__file__).parent / "ostrich-blended"
+    templates = tuple(_p.glob("model/*.rv?")) + tuple(_p.glob("*.t??"))
+
+    def __init__(self, *args, **kwds):
+        super().__init__(*args, **kwds)
+        self.rvi.suppress_output = True
+        self.txt = Ost(
+            algorithm="DDS",
+            max_iterations=50,
+            lowerBounds=BLENDED.params(
+                None,  # par_x01
+                None,  # par_x02
+                None,  # par_x03
+                None,  # 10**par_x04
+                None,  # par_x05
+                None,  # par_x06
+                None,  # par_x07
+                None,  # par_x08
+                None,  # par_x09
+                None,  # par_x09+par_x10
+                None,  # 10**par_x11
+                None,  # par_x12
+                None,  # par_x13
+                None,  # par_x13+par_x14
+                None,  # par_x15
+                None,  # par_x16
+                None,  # par_x17
+                None,  # par_x18
+                None,  # par_x19
+                None,  # par_x20
+                None,  # par_x21
+                None,  # par_x22
+                None,  # par_x23
+                None,  # par_x24
+                None,  # par_x24+par_x25
+                None,  # par_x26
+                None,  # par_x27
+                None,  # par_x28
+                None,  # par_x29
+                None,  # par_x30
+                None,  # par_x31
+                None,  # par_x32
+                None,  # par_x33
+                None,  # par_x34
+                None,  # par_x35
+                None,  # par_r01
+                None,  # par_r02
+                None,  # par_r03
+                None,  # par_r04
+                None,  # par_r05
+                None,  # par_r06
+                None,  # par_r07
+                None,  # par_r08
+            ),
+            upperBounds=BLENDED.params(
+                None,  # par_x01
+                None,  # par_x02
+                None,  # par_x03
+                None,  # 10**par_x04
+                None,  # par_x05
+                None,  # par_x06
+                None,  # par_x07
+                None,  # par_x08
+                None,  # par_x09
+                None,  # par_x09+par_x10
+                None,  # 10**par_x11
+                None,  # par_x12
+                None,  # par_x13
+                None,  # par_x13+par_x14
+                None,  # par_x15
+                None,  # par_x16
+                None,  # par_x17
+                None,  # par_x18
+                None,  # par_x19
+                None,  # par_x20
+                None,  # par_x21
+                None,  # par_x22
+                None,  # par_x23
+                None,  # par_x24
+                None,  # par_x24+par_x25
+                None,  # par_x26
+                None,  # par_x27
+                None,  # par_x28
+                None,  # par_x29
+                None,  # par_x30
+                None,  # par_x31
+                None,  # par_x32
+                None,  # par_x33
+                None,  # par_x34
+                None,  # par_x35
+                None,  # par_r01
+                None,  # par_r02
+                None,  # par_r03
+                None,  # par_r04
+                None,  # par_r05
+                None,  # par_r06
+                None,  # par_r07
+                None,  # par_r08
+            ),
+        )
+
+    def derived_parameters(self):
+        """Derived parameters are computed by Ostrich."""
+        pass
+
+    def ost2raven(self, ops):
+        """Return a list of parameter names calibrated by Ostrich that match Raven's parameters.
+
+        Parameters
+        ----------
+        ops: dict
+          Optimal parameter set returned by Ostrich.
+
+        Returns
+        -------
+        BLENDEDParams named tuple
+          Parameters expected by Raven.
+        """
+        names = ["par_x{:02}".format(i) for i in range(1, 36)] + [
+            "par_r{:02}".format(i) for i in range(1, 9)
+        ]
+        names[3] = "pow_x04"
+        names[9] = "sum_x09_x10"
+        names[10] = "pow_x11"
+        names[13] = "sum_x13_x14"
+        names[24] = "sum_x24_x25"
+
+        out = [ops[n] for n in names]
+        return self.params(*out)
+
+
+class Routing(Raven):
+    """Routing model - no hydrological modeling"""
+
+    identifier = "routing"
+    templates = tuple((Path(__file__).parent / "raven-routing").glob("*.rv?"))
+
+    params = namedtuple("RoutingParams", ())
+
+    def __init__(self, *args, **kwds):
+        super().__init__(*args, **kwds)
+
+        # Declare the parameters that can be user-modified.
+
+        self.rvi = RVI()
+        self.rvp = RVP()
+        self.rvh = RVH()
+        self.rvt = RVT()
+
+    def derived_parameters(self):
+        pass
+
+        # TODO: Compute this from the input file
+        # self.rvd["avg_annual_runoff"] = ?
+
+
 def get_model(name):
     """Return the corresponding Raven emulated model instance.
 
@@ -439,7 +699,7 @@ def get_model(name):
     model_cls = getattr(emulators, name, None)
 
     if model_cls is None:
-        for m in [GR4JCN, MOHYSE, HMETS, HBVEC]:
+        for m in [GR4JCN, MOHYSE, HMETS, HBVEC, BLENDED]:
             if m.identifier == name:
                 model_cls = m
 
