@@ -1,13 +1,37 @@
 import collections
 import datetime as dt
+from collections import namedtuple
+from dataclasses import dataclass
 from pathlib import Path
+from textwrap import dedent
+from typing import Dict, List, Tuple
 
 import cftime
-import geopandas
 import six
-from xclim.core.units import units, units2pint
+from xclim.core.units import units2pint
 
-from .state import BasinStateVariables, HRUStateVariables
+from .commands import (
+    BasinIndexCommand,
+    BasinStateVariablesCommand,
+    ChannelProfileCommand,
+    HRUsCommand,
+    HRUStateVariableTableCommand,
+    LandUseClassesCommand,
+    RainCorrection,
+    RavenConfig,
+    ReservoirCommand,
+    SBGroupPropertyMultiplierCommand,
+    SnowCorrection,
+    SoilClassesCommand,
+    SoilProfilesCommand,
+    SubBasinGroupCommand,
+    SubBasinsCommand,
+    VegetationClassesCommand,
+)
+
+HRU = HRUsCommand.Record
+HRUState = HRUStateVariableTableCommand.Record
+LU = LandUseClassesCommand.Record
 
 """
 Raven configuration
@@ -136,7 +160,7 @@ class RVFile:
         return pattern.findall(self.content)
 
 
-class RV(collections.abc.Mapping):
+class RV(collections.abc.Mapping, RavenConfig):
     """Generic configuration class.
 
     RV provides two mechanisms to set values, a dictionary-like interface and an object-like interface::
@@ -179,7 +203,17 @@ class RV(collections.abc.Mapping):
         return iter(self.keys())
 
     def keys(self):
-        return (key[1:] if key.startswith("_") else key for key in self.__dict__)
+        # Attributes
+        a = list(filter(lambda x: not x.startswith("_"), self.__dict__))
+
+        # Properties
+        p = list(
+            filter(
+                lambda x: isinstance(getattr(self.__class__, x, None), property),
+                dir(self),
+            )
+        )
+        return a + p
 
     def items(self):
         for attribute in self.keys():
@@ -395,9 +429,12 @@ class MonthlyAverage(RV):
 
 
 class RVT(RV):
-    def __init__(self, gridded_forcing_cmds=None, **kwargs):
+    def __init__(self, **kwargs):
         self._nc_index = None
-        self._gridded_forcings = gridded_forcing_cmds or []
+        self.gridded_forcings = ()
+        self.raincorrection = 1
+        self.snowcorrection = 1
+
         super(RVT, self).__init__(**kwargs)
 
     @property
@@ -411,8 +448,21 @@ class RVT(RV):
                 setattr(val, "index", value)
 
     @property
-    def gridded_forcings(self):
-        return self._gridded_forcings
+    def gridded_forcing_list(self):
+        return "\n\n".join(map(str, self.gridded_forcings))
+        # return GriddedForcingList(self.gridded_forcings)
+
+    @gridded_forcing_list.setter
+    def gridded_forcing_list(self, value):
+        self.gridded_forcings = value
+
+    @property
+    def raincorrection_cmd(self):
+        return RainCorrection(self.raincorrection)
+
+    @property
+    def snowcorrection_cmd(self):
+        return SnowCorrection(self.snowcorrection)
 
     def update(self, items, force=False):
         """Update values from dictionary items.
@@ -431,15 +481,6 @@ class RVT(RV):
                 if isinstance(val, dict):
                     self[key].update(val, force=True)
 
-    def to_rv(self):
-        return """
-{gridded_forcing_cmds}
-        """.format(
-            gridded_forcing_cmds="\n\n".join(
-                [gf.to_rv() for gf in self._gridded_forcings]
-            ),
-        )
-
 
 class RVI(RV):
     def __init__(self, **kwargs):
@@ -650,15 +691,15 @@ class RVI(RV):
 
 
 class RVC(RV):
-    def __init__(self, **kwargs):
-        self._hru_state = {}
-        self._basin_state = {}
-
-        # This is a hack to make sure the txt_hru_state and txt_basin_state are picked up to fill the rv templates.
-        self._txt_hru_state = ""
-        self._txt_basin_state = ""
-
-        super().__init__(**kwargs)
+    def __init__(
+        self,
+        hru_states: Dict[int, HRUState] = None,
+        basin_states: Dict[int, BasinIndexCommand] = None,
+        **kwds,
+    ):
+        self.hru_states = hru_states or {}
+        self.basin_states = basin_states or {}
+        super().__init__(**kwds)
 
     def parse(self, rvc):
         """Set initial conditions based on *solution* output file.
@@ -669,134 +710,118 @@ class RVC(RV):
           `solution.rvc` content.
         """
         objs = parse_solution(rvc)
-        hru, basin = get_states(objs)
-        self._hru_state.update(hru)
-        self._basin_state.update(basin)
+        self.hru_states, self.basin_states = get_states(objs)
 
     @property
     def hru_state(self):
-        return self._hru_state.get(1)
+        return self.hru_states.get(1, None)
 
     @hru_state.setter
     def hru_state(self, value):
-        self._hru_state[1] = value
+        self.hru_states[1] = value
 
     @property
     def basin_state(self):
-        return self._basin_state.get(1)
+        return self.basin_states.get(1, None)
 
     @basin_state.setter
     def basin_state(self, value):
-        self._basin_state[1] = value
+        self.basin_states[1] = value
 
     @property
-    def txt_hru_state(self):
+    def hru_states_cmd(self):
         """Return HRU state values."""
-        txt = []
-        for index, data in self._hru_state.items():
-            txt.append(f"{index}," + ",".join(map(repr, data)))
-
-        return "\n".join(txt)
+        return HRUStateVariableTableCommand(self.hru_states)
 
     @property
-    def txt_basin_state(self):
+    def basin_states_cmd(self):
         """Return basin state variables."""
-        pat = """
-              :BasinIndex {index},{name}
-                :ChannelStorage, {channelstorage}
-                :RivuletStorage, {rivuletstorage}
-                :Qout,{nsegs},{qout},{qoutlast}
-                :Qlat,{nQlatHist},{qlat},{qlatlast}
-                :Qin ,{nQinHist}, {qin}
-            """
-        txt = []
-        for index, data in self._basin_state.items():
-            txt.append(
-                pat.format(
-                    **data._asdict(),
-                    nsegs=len(data.qout),
-                    nQlatHist=len(data.qlat),
-                    nQinHist=len(data.qin),
-                )
-            )
-        return "\n".join(txt).replace("[", "").replace("]", "")
+        return BasinStateVariablesCommand(self.basin_states)
 
 
+@dataclass
 class RVH(RV):
-    def __init__(
-        self,
-        subbasins_cmd,
-        land_subbasin_group_cmd,
-        lake_subbasin_group_cmd,
-        reservoir_cmds,
-        hrus_cmd,
-        **kwargs,
-    ):
-        self._subbasins_cmd = subbasins_cmd
-        self._land_subbasin_group_cmd = land_subbasin_group_cmd
-        self._lake_subbasin_group_cmd = lake_subbasin_group_cmd
-        self._reservoir_cmds = reservoir_cmds
-        self._hrus_cmd = hrus_cmd
+    subbasins: Tuple[SubBasinsCommand.Record] = (SubBasinsCommand.Record(),)
+    land_subbasins: Tuple[int] = ()
+    lake_subbasins: Tuple[int] = ()
+    reservoirs: Tuple[ReservoirCommand] = ()
+    hrus: Tuple[HRUsCommand.Record] = ()
+    sb_group_property_multipliers: Tuple[SBGroupPropertyMultiplierCommand] = ()
 
-        super().__init__(**kwargs)
+    template = """
+    {subbasins_cmd}
+
+    {hrus_cmd}
+
+    {land_subbasin_group_cmd}
+
+    {lake_subbasin_group_cmd}
+
+    {reservoir_cmd_list}
+    """
 
     @property
-    def subbasins(self):
-        return self._subbasins_cmd.subbasins  # return internal list
+    def subbasins_cmd(self):
+        return SubBasinsCommand(self.subbasins)
 
     @property
-    def land_subbasin_group(self):
-        return self._land_subbasin_group_cmd.subbasin_ids  # return internal list
+    def land_subbasin_group_cmd(self):
+        return SubBasinGroupCommand("Land", self.land_subbasins)
 
     @property
-    def lake_subbasin_group(self):
-        return self._lake_subbasin_group_cmd.subbasin_ids  # return internal list
+    def lake_subbasin_group_cmd(self):
+        return SubBasinGroupCommand("Lakes", self.lake_subbasins)
 
     @property
-    def reservoirs(self):
-        return self._reservoir_cmds  # return list directly
+    def reservoir_cmd_list(self):
+        return "\n\n".join(map(str, self.reservoirs))
 
     @property
-    def hrus(self):
-        return self._hrus_cmd.hrus  # return internal list
+    def hrus_cmd(self):
+        return HRUsCommand(self.hrus)
+
+    @property
+    def sb_group_property_multiplier_list(self):
+        return "\n\n".join(map(str, self.sb_group_property_multipliers))
 
     def to_rv(self):
-        return """
-{subbasins_cmd}
-
-{hrus_cmd}
-
-{land_subbasin_group_cmd}
-
-{lake_subbasin_group_cmd}
-
-{reservoir_cmds}
-        """.format(
-            subbasins_cmd=self._subbasins_cmd.to_rv(),
-            hrus_cmd=self._hrus_cmd.to_rv(),
-            land_subbasin_group_cmd=self._land_subbasin_group_cmd.to_rv(),
-            lake_subbasin_group_cmd=self._lake_subbasin_group_cmd.to_rv(),
-            reservoir_cmds="\n\n".join([r.to_rv() for r in self._reservoir_cmds]),
-        )
+        # params = self.items()
+        return dedent(self.template).format(**dict(self.items()))
 
 
+@dataclass
 class RVP(RV):
-    def __init__(self, channel_profile_cmds, **kwargs):
-        self._channel_profile_cmds = channel_profile_cmds
-        super().__init__(**kwargs)
+    params: Tuple[namedtuple] = ()
+    soil_classes: Tuple[SoilClassesCommand.Record] = ()
+    soil_profiles: Tuple[SoilProfilesCommand.Record] = ()
+    vegetation_classes: Tuple[VegetationClassesCommand.Record] = ()
+    land_use_classes: Tuple[LandUseClassesCommand.Record] = ()
+    channel_profiles: Tuple[ChannelProfileCommand] = ()
 
     @property
-    def channel_profiles(self):
-        return self._channel_profile_cmds
+    def soil_classes_cmd(self):
+        return SoilClassesCommand(self.soil_classes)
 
-    def to_rv(self):
-        return """
-{channel_profile_cmds}
-        """.format(
-            channel_profile_cmds="\n\n".join(
-                [cp.to_rv() for cp in self._channel_profile_cmds]
-            ),
-        )
+    @property
+    def soil_profiles_cmd(self):
+        return SoilProfilesCommand(self.soil_profiles)
+
+    @property
+    def vegetation_classes_cmd(self):
+        return VegetationClassesCommand(self.vegetation_classes)
+
+    @property
+    def land_use_classes_cmd(self):
+        return LandUseClassesCommand(self.land_use_classes)
+
+    @property
+    def channel_profile_cmd_list(self):
+        return "\n\n".join(map(str, self.channel_profiles))
+
+    # Note sure about this!
+    # def to_rv(self):
+    #     params = self.items()
+    #     return dedent(self.template).format(**dict(self.items()))
 
 
 class Ost(RV):
@@ -869,11 +894,11 @@ def get_states(solution, hru_index=None, basin_index=None):
     basin_state = {}
 
     for index, params in solution["HRUStateVariableTable"]["data"].items():
-        hru_state[index] = HRUStateVariables(*params)
+        hru_state[index] = HRUState(*params)
 
     for index, raw in solution["BasinStateVariables"]["BasinIndex"].items():
         params = {k.lower(): v for (k, v) in raw.items()}
-        basin_state[index] = BasinStateVariables(**params)
+        basin_state[index] = BasinIndexCommand(**params)
 
     if hru_index is not None:
         hru_state = hru_state[hru_index]
@@ -937,6 +962,6 @@ def _parser(lines, indent="", fmt=str):
         else:
             data = line.split(",")
             i = int(data.pop(0))
-            out["data"][i] = list(map(float, data))
+            out["data"][i] = [i] + list(map(float, data))
 
     return out
