@@ -5,7 +5,7 @@ import pandas as pd
 import xarray as xr
 
 
-def count_pixels(stats, numeric_categories=False):
+def count_pixels(stats: dict, numeric_categories=False) -> int:
     category_counts = 0
     for key, val in stats.items():
         if numeric_categories:
