@@ -1,18 +1,13 @@
 import datetime as dt
-import os
-import pdb
-import tempfile
-from copy import deepcopy
+from dataclasses import replace
 
 import matplotlib.pyplot as plt
 import numpy as np
-import pandas as pd
-import pytest
 import xarray as xr
 
 from ravenpy.models import GR4JCN
+from ravenpy.models.commands import BasinIndexCommand
 from ravenpy.models.rv import RVC
-from ravenpy.models.state import BasinStateVariables
 from ravenpy.utilities.data_assimilation import assimilate, perturbation
 from ravenpy.utilities.testdata import get_local_testdata
 
@@ -160,7 +155,7 @@ class TestAssimilationGR4JCN:
             # Get new initial conditions and feed assimilated values
             hru_states, basin_states = model.get_final_state()
             hru_states = [
-                hru_states[i]._replace(**dict(zip(assim_var, xa[:, i])))
+                replace(hru_states[i], **dict(zip(assim_var, xa[:, i])))
                 for i in range(n_members)
             ]
 
@@ -170,12 +165,8 @@ class TestAssimilationGR4JCN:
         model.rvi.run_name = "ref"
         model.rvi.start_date = start_date
         model.rvi.end_date = end_date
-        model.rvc = RVC(soil0=None, soil1=15, basin_state=BasinStateVariables())
-        model(
-            [
-                ts,
-            ]
-        )
+        model.rvc = RVC(soil0=None, soil1=15, basin_state=BasinIndexCommand())
+        model([ts])
 
         # We can now plot everything!
         plt.plot(q_assim.T, "r", label="Assimilated")  # plot the assimilated flows
